
#include "uassert.h"

#define MALFORMED_INPUT goto finish

#define GET_BYTE                                 \
    ch = *us++;                                  \
    if (ch < 0x80 || 0xBF < ch) MALFORMED_INPUT; \
    result = (result << 6) | (ch & ~0x80);

/* Scan a UTF-8 character from the given "s", advancing the
 * pointer and storing its new value in "ps" (if non-NULL).
 * If the first byte of "s" is invalid, then scan that one byte.
 * If the first byte of "s" indicates a multi-byte sequence
 * but one of the later bytes is invalid, then scan up to the
 * offending byte and leave the offending byte unscanned.
 * For example, plainutf8_scan("\xC0\x80", &s) advances s to
 * point at the "\x80", rather than risking an infinite loop;
 * and plainutf8_scan("\xD2", &s) leaves s pointing at the
 * terminating '\0', rather than scanning the '\0' and leaving
 * s pointing outside the input buffer.
 */
uint32 plainutf8_scan(const utf8_char *s, void *ps)
{
    const unsigned char *us = (const unsigned char *)s;
    unsigned char ch;
    uint32 result = 0;
    assert(us != NULL);

    result = *us++;
    if (result <= 0x7F) {
        /* continue */
    } else if (result <= 0xC1) {
        MALFORMED_INPUT;
    } else if (result <= 0xDF) {
        /* Two-byte sequence. */
        result &= ~0xC0;
        GET_BYTE
    } else if (result <= 0xEF) {
        /* Three-byte sequence. */
        result &= ~0xE0;
        GET_BYTE
        GET_BYTE
    } else if (result <= 0xF7) {
        /* Four-byte sequence. */
        result &= ~0xF0;
        GET_BYTE GET_BYTE
        GET_BYTE
    } else if (result <= 0xFB) {
        /* Five-byte sequence. */
        result &= ~0xF8;
        GET_BYTE GET_BYTE
        GET_BYTE GET_BYTE
    } else if (result <= 0xFB) {
        /* Six-byte sequence. */
        result &= ~0xFC;
        GET_BYTE GET_BYTE
        GET_BYTE GET_BYTE
        GET_BYTE
    } else {
        MALFORMED_INPUT;
    }
  finish:
    if (ps != NULL) *(const utf8_char **)ps = (const utf8_char *)us;
    return result;
}

