
#include "uassert.h"
#include <string.h>  /* memcpy */

/*
 * DESCRIPTION
 * The strcat() function appends the src string to the dest string,
 * overwriting the null byte ('\0') at the end of dest, and then adds a
 * terminating null byte. The strings may not overlap, and the dest string
 * must have enough space for the result.
 *
 * The strncat() function is similar, except that
 *   * it will use at most n characters from src; and
 *   * src does not need to be null terminated if it contains n or more
 *   characters.
 *
 * As with strcat(), the resulting string in dest is always null terminated.
 *
 * RETURN VALUE
 * The strcat() and strncat() functions return a pointer to the resulting
 * string dest.
 */

utf8_char *utf8_strncat(utf8_char *dest, const utf8_char *src, size_t n)
{
    /* The utf8_strncat function takes a count of UTF-8 characters "n",
     * examines "src" to figure out how many bytes that will be, and
     * then copies those bytes as strncat() would.
     */
    const utf8_char *endsrc = src;
    utf8_char *enddest = dest;
    size_t count;
    
    assert(dest != NULL);
    assert(src != NULL);
    
    if (n == 0)
      return dest;

    /* Find the end of "dest", where we are to append "src". */
    while (*enddest != '\0')
      (void)plainutf8_scan(enddest, &enddest);
    
    for (count = 0; count < n && *endsrc != '\0'; ++count)
      (void)plainutf8_scan(endsrc, &endsrc);
    
    memcpy(enddest, src, endsrc-src);
    enddest[endsrc-src] = '\0';
    return dest;
}

