
#include "uassert.h"

/*
 * DESCRIPTION
 * The strcmp() function compares the two strings s1 and s2. It returns
 * an integer less than, equal to, or greater than zero if s1 is found,
 * respectively, to be less than, to match, or be greater than s2.
 * The strncmp() function is similar, except it only compares the first
 * (at most) n characters of s1 and s2.
 * RETURN VALUE
 * The strcmp() and strncmp() functions return an integer less than, equal
 * to, or greater than zero if s1 (or the first n bytes thereof) is found,
 * respectively, to be less than, to match, or be greater than s2.
 */

int utf8_strncmp(const utf8_char *s1, const utf8_char *s2, size_t n)
{
    /* Note that plain old strcmp() on UTF-8 strings yields the proper
     * ordering; so you might think we're doing extra work by decoding
     * each string into "uint32" values and comparing those. However,
     * since we have to count UTF-8 characters as we go, this way is
     * much easier and faster than trying to do something tricky with
     * the raw bytes.
     */
    size_t count = 0;

    assert(s1 != NULL);
    assert(s2 != NULL);
    
    for (count = 0; count < n && *s1 != '\0' && *s2 != '\0'; ++count) {
        const uint32 c1 = plainutf8_scan(s1, &s1);
        const uint32 c2 = plainutf8_scan(s2, &s2);
        if (c1 < c2) return -1;
        if (c1 > c2) return 1;
    }
    return 0;
}

