
#include "uassert.h"
#include <string.h>  /* memcpy, memset */

/*
 * DESCRIPTION
 * The strcpy() function copies the string pointed to by src, including
 * the terminating null byte ('\0'), to the buffer pointed to by dest.
 * The strings may not overlap, and the destination string dest must be
 * large enough to receive the copy.
 *
 * The strncpy() function is similar, except that at most n bytes of src
 * are copied.  Warning: If there is no null byte among the first n bytes
 * of src, the string placed in dest will not be null-terminated.
 *
 * If the length of src is less than n, strncpy() pads the remainder of
 * dest with null bytes.
 *
 * RETURN VALUE
 * The strncpy() function returns a pointer to the destination
 * string dest.
 */

utf8_char *utf8_strncpy(utf8_char *dest, const utf8_char *src, size_t n)
{
    /* The utf8_strncpy function takes a count of UTF-8 characters "n",
     * examines "src" to figure out how many bytes that will be, and
     * then copies those bytes as strncpy() would.
     * If there is no '\0' among the first n UTF-8-encoded characters
     * of "src", the string placed in "dest" will not be null-terminated.
     * This is not very useful, but then this entire function is rather
     * dangerous. Note that utf8_strncpy behaves identically to strncpy
     * as long as "src" does not contain any non-ASCII characters.
     */
    const utf8_char *end = src;
    size_t count;
    
    assert(dest != NULL);
    assert(src != NULL);
    
    if (n == 0)
      return dest;
    
    for (count = 0; count < n && *end != '\0'; ++count)
      (void)plainutf8_scan(end, &end);
    memcpy(dest, src, end-src);
    if (count < n)
      memset(&dest[end-src], '\0', n-count);
    return dest;
}

