
#include "uassert.h"

/*
 * DESCRIPTION
 * The strpbrk() function locates the first occurrence in the string s of
 * any of the characters in the string accept.
 * RETURN VALUE
 * The strpbrk() function returns a pointer to the character in s that
 * matches one of the characters in accept, or NULL if no such character
 * is found.
 */

utf8_char *utf8_strpbrk(const utf8_char *s, const utf8_char *accept)
{
    assert(s != NULL);
    assert(accept != NULL);

    while (*s != '\0') {
        const utf8_char *new_s;
        const uint32 ch32 = plainutf8_scan(s, &new_s);
        const utf8_char *needle;
        for (needle = accept; *needle != '\0'; ) {
            if (ch32 == plainutf8_scan(needle, &needle))
              return (utf8_char *)s;
        }
        /* This "ch32" was not found in "accept". */
        s = new_s;
    }
    return NULL;
}

