
#include "uassert.h"

/*
 * DESCRIPTION
 * If *stringp is NULL, the strsep() function returns NULL and does nothing
 * else. Otherwise, this function finds the first token in the string
 * *stringp, where tokens are delimited by symbols in the string delim.
 * This token is terminated with a '\0' character (by overwriting the
 * delimiter) and *stringp is updated to point past the token. In case no
 * delimiter was found, the token is taken to be the entire string
 * *stringp, and *stringp is made NULL.
 * RETURN VALUE
 * The strsep() function returns a pointer to the token, that is, it
 * returns the original value of *stringp.
 */

utf8_char *utf8_strsep(utf8_char **stringp, const utf8_char *delim)
{
    utf8_char *s;
    utf8_char *next_s = NULL;
    uint32 ch32;

    assert(stringp != NULL);
    assert(delim != NULL);

    s = *stringp;
    if (s == NULL) {
        return NULL;
    }
    
    while (*s != '\0') {
        const utf8_char *needle;
        ch32 = plainutf8_scan(s, &next_s);
        for (needle = delim; *needle != '\0'; ) {
            if (ch32 == plainutf8_scan(needle, &needle)) {
                /* This character is a delimiter. */
                *s = '\0';
                goto finish;
            }
        }
        /* No delimiter has yet been found. */
        s = next_s;
    }
    /* No delimiter was ever found. Let *stringp be NULL on return. */
    next_s = NULL;

  finish:
    /* Return the original value of *stringp. */
    s = *stringp;
    *stringp = next_s;
    return s;
}

