
#include "uassert.h"
#include <string.h>  /* memcpy */

extern int utf8_wctomb(utf8_char *s, wchar_t wc);

/*
 * DESCRIPTION
 * If dest is not a NULL pointer, the wcstombs() function converts the
 * wide-character string src to a multibyte string starting at dest. At
 * most n bytes are written to dest. The conversion starts in the initial
 * state. The conversion can stop for three reasons:
 *
 * 1. A wide character has been encountered that cannot be represented as
 *    a multibyte sequence (according to the current locale).  In this case
 *    (size_t) -1 is returned.
 *
 * 2. The length limit forces a stop. In this case the number of bytes
 *    written to dest is returned, but the shift state at this point is lost.
 *
 * 3. The wide-character string has been completely converted, including
 *    the terminating L'\0'. In this case the conversion ends in the initial
 *    state. The number of bytes written to dest, excluding the terminating
 *    '\0' byte, is returned.
 *
 * The programmer must ensure that there is room for at least n bytes at
 * dest.
 *
 * If dest is NULL, n is ignored, and the conversion proceeds as above,
 * except that the converted bytes are not written out to memory, and that
 * no length limit exists.
 *
 * RETURN VALUE
 * The wcstombs() function returns the number of bytes that make up the
 * converted part of the multibyte sequence, not including the terminating
 * null byte. If a wide character was encountered which could not be converted,
 * (size_t) -1 is returned.
 */

size_t utf8_wcstombs(utf8_char *dest, const wchar_t *src, size_t n)
{
    size_t bytes_written = 0;

    assert(n == 0 || dest != NULL);
    assert(src != NULL);

    while (bytes_written < n && *src != '\0') {
        utf8_char temp[8];
        const int count = utf8_wctomb(temp, *src);
        if (count == -1) {
            /* Case 1. The wchar_t cannot be converted. */
            return (size_t) -1;
        } else if (bytes_written + count > n) {
            /* Case 2. */
            return bytes_written;
        }
        assert(count > 0);
        assert(count <= 6);
        memcpy(dest, temp, count);
        dest += count;
	bytes_written += count;
        ++src;
    }
    if (bytes_written <= n) {
        /* Case 3. */
        dest[bytes_written] = '\0';
    }
    return bytes_written;
}

